/****************************************************  Channel Functions  *****************************************************

	Red Rapids
	797 North Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	The following functions are specific to individual SigFPGA channels.

	Rev History:  11-28-2018:	Style and format changes.
				  08-08-2018:	Inital release.

	Copyright 2018, Red Rapids, Inc.

****************************************************  Channel Functions  *****************************************************/

#include "driver_functions.h"
#include "sigfpga.h"
#include "channel_functions.h"

// Set the channel interrupt mask bits to enable (1) or disable (0) interrupt generation for each event type.
void ChannelMask(s_Handle *p_HW, unsigned ChannelNumber, s_ChannelIntMask ChannelIntMask)
{
	// Write to the channel interrupt mask address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + CHANNEL_INT_MASK] =
				(unsigned long long)ChannelIntMask.DMAMarker |
				(unsigned long long)ChannelIntMask.Error[0] << 4 |
				(unsigned long long)ChannelIntMask.Error[1] << 5;
}

// Clear the channel status register.
void ClearChannelStatus(s_Handle *p_HW, unsigned ChannelNumber)
{
	// Write to the channel status address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + CHANNEL_STATUS] = 0x0ull;
}

// Enable (OnOff = 1) or disable (OnOff = 0) channel processing.
void FlowControl(s_Handle *p_HW, unsigned ChannelNumber, unsigned OnOff)
{
	// Write to the synchronizer control address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + SYNCHRONIZER_CTL] = (unsigned long long)OnOff;
}

// Report the current on/off status of the requested channel.
// Return of zero indicates the channel is off, one indicates the channel is on.
int FlowStatus(s_Handle *p_HW, unsigned ChannelNumber)
{
	// Read the synchronizer control address.
	return (int)(p_HW->DEV.BAR2[(512 * ChannelNumber) + SYNCHRONIZER_CTL] & 0x0000000000000001ull);
}

// Report the channel status and clear the values.
// Returns the total number of status events detected.
int GetChannelStatus(s_Handle *p_HW, unsigned ChannelNumber, s_ChannelStatus *p_ChannelStatus)
{
	unsigned long long value = 0;
	int status = 0;
	unsigned cnt = 0;
	
	// Save and clear the contents of the channel status register by writing to the error status address.
	// Note: The contents are saved to a shadow register that is accessed during a read operation (below).
	p_HW->DEV.BAR2[(512 * ChannelNumber) + CHANNEL_STATUS] = 0x0ull;
	// Read the saved contents of the channel status.
	value = p_HW->DEV.BAR2[(512 * ChannelNumber) + CHANNEL_STATUS];

	// Strip the DMA marker bit.
	if ((p_ChannelStatus->DMAMarker = value & 0x0000000000000001ull)) status++;

	// Strip the individual channel error indicator bits.
	for (cnt = 0; cnt < 2; cnt++)
		if ((p_ChannelStatus->Error[cnt] = value >> ((cnt + 4)) & 0x0000000000000001ull)) status++;

	// Return the total fault count.
	return status;
}

// Configure the synchronizer operating controls.
void LoadSynchronizer(s_Handle *p_HW, unsigned ChannelNumber, s_Synchronizer Synchronizer)
{
	// Write to the synchronizer command address.
	p_HW->DEV.BAR2[(512 * ChannelNumber) + SYNCHRONIZER_CMD] =
			(unsigned long long)Synchronizer.StartEvent |
			(unsigned long long)Synchronizer.StopEvent << 2 |
			(unsigned long long)Synchronizer.SampleCount << 32;
}

// Report the current ready status of the requested channel.
// Return of zero indicates the channel is not ready for a start trigger, one indicates the channel is ready.
int ReadyStatus(s_Handle *p_HW, unsigned ChannelNumber)
{
	// Read the synchronizer control address.
	// Note: Shift the ready status bit so that the function returns either one or zero.
	return (int)((p_HW->DEV.BAR2[(512 * ChannelNumber) + SYNCHRONIZER_CTL] >> 1) & 0x0000000000000001ull);
}

